<?php

namespace app\admin\controller;

use app\common\model\ai\Config;

class Aiconfig extends Botbase
{
    /**
     * @var Config
     */
    protected $model;
    /**
     * @var array
     */
    private $tabList;
    /**
     * @var array
     */
    private $aiDrivers;

    public function initialize()
    {
        parent::initialize(); // TODO: Change the autogenerated stub
        $this->model = new Config();
        $this->aiDrivers = [
            'aiedu' => 'AIEdu（gpt国内版）',
            'weixin' => '微信AI',
            'qyk' => '青云客'
        ];
        $this->tabList = [
            'basic' => ['title' => '基础设置', 'href' => url('index', ['name' => 'basic'])],
            'driver' => ['title' => '驱动设置', 'href' => url('index', ['name' => 'driver'])]
        ];
    }

    /**
     * 设置
     * @return mixed
     * @throws \think\Exception
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function index(){
        $name = input('name', 'basic');
        $tip = "<ul><li>AIEdu（ChatGpt-3.5）: <a href='https://leyoyu.cn/' target='_blank'>https://leyoyu.cn/</a></li>
<li>微信OpenAi: <a target='_blank' href='https://openai.weixin.qq.com/'>https://openai.weixin.qq.com/</a></li>
<li>青云客Ai: <a target='_blank' href='http://api.qingyunke.com/'>http://api.qingyunke.com/</a></li>
</ul>";
        $settings = $this->model->getConf(['bot_id' => $this->bot['id']]);
        if(!empty($settings['wxids'])){
            $settings['wxids'] = explode(',', $settings['wxids']);
        }else{
            $settings['wxids'] = [];
        }
        if(empty($settings)){
            $settings = [
                'driver' => 'aiedu',
                'switch' => 1,
                'need_at' => 1,
                'show_question' => 1,
                'context_length' => 10
            ];
        }
        if(empty($settings['content_rule'])){
            $settings['content_rule'] = "1、不允许回答任何政治、色情等一些列不符合中国法律法规的问题 \n 2. 请以markdown的格式返回答案";
        }

        $builder = new FormBuilder();
        $builder->setPostUrl(url('savePost'))
            ->setTabNav($this->tabList, $name)
            ->addFormItem('id', 'hidden', 'id', 'id');
            switch ($name){
                case 'driver':
                    $builder->setTip($tip)
                        ->addFormItem('aiedu_legend', 'legend', 'AiEdu(Base ChatGPT-3.5)','AiEdu' )
                        ->addFormItem('aiedu_key', 'text', 'Key', 'AiEdu的key,自行到https://pandapy.com/ 获取')
                        ->addFormItem('weixin_legend', 'legend', '微信智能对话','微信智能对话' )
                        ->addFormItem('wx_appid', 'text', 'Appid', '微信智能对话的Appid')
                        ->addFormItem('wx_token', 'text', 'Token', '微信智能对话的Token')
                        ->addFormItem('wx_encoding_aes_key', 'text', 'EncodingAesKey', '微信智能对话的EncodingAesKey')
                        ->addFormItem('qyk_legend', 'legend', '青云客','青云客' )
                        ->addFormItem('qyk_appid', 'text', 'Appid', '填0就好');
                    break;
                default:
                    $builder->addFormItem('switch', 'radio', '开启', '是否开启', [1=>'是', 0 => '否'], 'required')
                        ->addFormItem('time_on', 'time', '上班时间', '最早的时间', [], 'required')
                        ->addFormItem('time_off', 'time', '下班时间', '晚上最迟', [], 'required')
                        ->addFormItem('driver', 'radio', 'AI驱动', 'AI驱动', $this->aiDrivers)
                        ->addFormItem('content_rule', 'textarea', '内容规范', '例如法律法规、输出内容格式等', [], 'maxlength=1000 style=height:100px;')
                        ->addFormItem('character_design', 'textarea', '助手人设', '如果你想用作客服，这个助手人设就非常必要了。助手人设可以写一些助手介绍、公司介绍、业务介绍等。1000字内', [], 'maxlength=1000 style=height:200px;')
                        ->addFormItem('context_length', 'number', '上下文长度', '保留上下文长度可以帮助AI记忆对话，金阿姨设置2-10之间', [], 'max=10')
                        ->addFormItem('wxids', 'chosen_multi', '作用对象', '指定的作用对象才会生效', $this->getMembers(), 'required')
                        ->addFormItem('need_at', 'radio', '被@后触发', '群聊中是否需要被at才回答', [1=>'是', 0 => '否'])
                        ->addFormItem('show_question', 'radio', '答案是否带上问题', '答案是否带上问题', [1=>'是', 0 => '否']);
                    break;
            }

        return $builder->setFormData($settings)->show();
    }

    public function savePost($jump_to = '/undefined', $data = [])
    {
        $jump_to = url('index');
        $post_data = input('post.');
        if(isset($post_data['wxids']) && empty($post_data['wxids'])){
            $this->error('请选择作用对象！');
        }
        unset($post_data['__token__']);
        foreach ($post_data as $k => $v){
            if($config = $this->model->getOneByMap(['bot_id' => $this->bot['id'], 'key' => $k],  true, true)){
                $this->model->updateOne(['id' => $config['id'], 'value' => $v]);
            }else{
                $this->model->addOne([
                    'admin_id' => $this->adminInfo['id'],
                    'bot_id' => $this->bot['id'],
                    'key' => $k,
                    'value'=> $v
                ]);
            }
        }
        $this->model->getConf(['bot_id' => $this->bot['id']], '', true);
        $this->success('数据保存成功', $jump_to);
    }
}